package tctec;

/*
 * Version: 26-July-2008
 * Simple Super4 java class based on jd2xx java ftdi interface
 */

import java.io.*;
import java.util.*;
import jd2xx.JD2XX;

import jd2xx.JD2XX.DeviceInfo;



public class super4 {
	
	/*
	 * List all connected tctec relay modules
	 * Returns them in an Arraylist of Strings
	 */
	public static ArrayList<String> listTCTECRelays()
	{
		ArrayList<String> returnArray = new ArrayList<String>() ;
		JD2XX jd = new JD2XX() ;
		
		try{
		String[] list = (String[]) jd.listDevicesBySerialNumber() ;
		
		
		for(int i = 0 ; i < list.length; i++)
		{
			JD2XX aJd = new JD2XX() ;
			aJd.openBySerialNumber(list[i]) ;
			
			DeviceInfo di = aJd.getDeviceInfo() ;
			if (di.description.contentEquals("TCTEC USB RELAY"))
			{
				System.out.printf("Found: %s\n", di.serial);
				returnArray.add(di.serial) ;
			}
			aJd.close() ;
		
		}
		}catch (IOException e)
		{
			System.out.print("IOException in listTCTECRelays()") ;
		
		}
		
		return(returnArray);
	
	}

	/*
	 * Set/unset the relays
	 */
	
	public static int setBits(String serialNumber, byte mask, int on )
	{
				
		JD2XX jd = new JD2XX();
		try{
			jd.openBySerialNumber(serialNumber);
			int bitMode = jd.getBitMode();
			bitMode >>= 4 ;
			
			if (bitMode > 0)
			{
				// the bit mode has not yet been set
				// so set it
				// only done once for every time the Super4 is plugged in 
				// 
				
				// first set to bitbang mode and all inputs (all off)
				//
				jd.setBitMode(0xF0, 0x01) ;
				
				// set to all high (off)
				//
				jd.write(0x0F) ;
				
				// set to outputs
				//
				jd.setBitMode(0xFF, 0x01) ;				
			}	
				
			// read the current state of the relays
			//
			int relayStates = jd.read();
			
			// invert states because 0 = on and 1 = off
			//
			relayStates = (~relayStates  & 0x0F);
			if (on == 1)
			{
				relayStates |= mask ;			
			}else
			{
				relayStates &= (~mask & 0x0F) ;
			
			}
			
			// set the relays, invert before writing out
			//
			relayStates =  (~relayStates & 0x0F) ;
			jd.write(relayStates) ;		
			jd.close() ;
		
		}catch(IOException e){
			System.out.println("IOException in setBits");			
		}	
		
		
		
	return(0);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		// list all the attached Super4 relay modules
		//
		ArrayList<String> boards = super4.listTCTECRelays() ;
		
		// for every Super4 relay board found, cycle through the bits
		//
		
		for(int bitCount = 0x0F ; bitCount >= 0x00; bitCount--)
		{
		  for(int i = 0 ; i < boards.size(); i++)
		  {			  
			  
			  // turn unset bits off
			  //
			  setBits(boards.get(i), (byte)(~bitCount & 0x0F), 0) ;
			  
			  // turn set bits on
			  //
			  setBits(boards.get(i), (byte)bitCount, 1) ;			
			  
			  // pause
			  //
			  try{
			  Thread.currentThread();
			Thread.sleep(500) ;
			  }catch(InterruptedException e){}
		
		  }
		}
		
		
		
				

	}

}
